#!/usr/local/bin/perl

    use strict;
    use lib '';
    use GT::Base;
    use GT::Plugins qw/STOP CONTINUE/;
    use Links qw/$CFG $IN $DB/;
    use LWP::Simple;
    use GT::Date;
    use Links::Plugins;
    use GT::CGI;
    use GT::SQL::Condition;
    use Getopt::Long;

   # Inherit from base class for debug and error methods
    @Plugins::AllPosters_Import::ISA = qw(GT::Base);

   # Change directory to our admin one...
    chdir(); 
    Links::init();

    my $opts = Links::Plugins->get_plugin_user_cfg('AllPosters_Import');

    my $ALLPOSTERSID = $opts->{'AffiliateID'};

    my $DEBUG = 0;    

    my ($grab_url,$category_root,$cleanup,$dograb);
    my $result = GetOptions ('file=s'  => \$grab_url,
                             'category=s' => \$category_root,
                             'clean=s'    => \$cleanup,
                             'dograb=s'   => \$dograb
                            );     

   # decide if we want to actually grab the files, or not...
    if ($dograb eq "yes") {
      system("rm -f *.txt");
      system("rm -f *.zip");
      system("wget -OAllPostersFeed.zip '$grab_url'");
      system("unzip AllPostersFeed.zip");
    } else {
      print qq{Skipping file downloads. Attempting to find the most current version of DataFeed files now.... \n};
    } 
    
    my $category_names_file;     # CategoryInformation...txt
    my $category_relations_file; # CategoryRelationship...txt
    my $category_prod_info_file; # CategoryProductInformation...,.txt
    my $image_details_file;      # ImageInformation....txt


    # grab a list of the .txt file... 
     my $file;
     opendir(DIR, "$CFG->{admin_root_path}/ALLPOSTERS");
      while (defined($file = readdir(DIR))) { 
         next if !defined $file;
         $file =~ s/ /~~/g;
         if ($file =~ /^CategoryInformation$category_root\_(.*)\.txt$/) { 
            $file =~ s/~~/ /g;
            $category_names_file = $file;
         }
         if ($file =~ /^CategoryRelationship$category_root\_(.*)\.txt$/) { 
            $file =~ s/~~/ /g;
            $category_relations_file = $file;
         }
         if ($file =~ /^CategoryProductInformation$category_root\_(.*)\.txt$/) { 
            $file =~ s/~~/ /g;
            $category_prod_info_file = $file;
         }
         if ($file =~ /^ImageInformation$category_root\_(.*)\.txt$/) { 
            $file =~ s/~~/ /g;
            $image_details_file = $file;
         }
      }
     closedir(DIR);


    print qq{ files:

       $category_names_file
       $category_relations_file
       $category_prod_info_file
       $image_details_file

    } if $DEBUG > 0;

   sleep 5;
    
   if ($cleanup eq "yes") {
    $DB->table('Links')->delete_all();
    $DB->table('Category')->delete_all();
    $DB->table('CatLinks')->delete_all();
   }
           
   # build up a "quick to use" hash, with cat names/numbers...
    my $count = 0;
    my $cat_names;
    open(NAMES, $category_names_file) || die "Unable to open $category_prod_info_file. Reason: $!";
      while (<NAMES>) {
       chomp;
       $count++;
       if ($count <= 1) { next; }
       my ($CategoryName,$CategoryNumber) = split /\|/, $_;
       $cat_names->{$CategoryNumber} = $CategoryName; 
      }
    close(CATSREAD);

    print "Loaded category_prod_info_file into memory...\n";

    #use Data::Dumper;
    #die Dumper($cat_names);

    my $count = 0;
    my $categories;
    open(CATSREAD, $category_relations_file) || die "Unable to open $category_relations_file. Reason: $!";
      while (<CATSREAD>) {
       chomp;
       $count++;
       if ($count <= 1) { next; }
       my ($CategoryNumber,$ParentCategoryNumber,$CategoryName) = split /\|/, $_;
    
       $categories->{$ParentCategoryNumber} .= $CategoryNumber . "-"; 

      }
    close(CATSREAD);

    print "Loaded $category_relations_file into memory...\n";
    sleep 5;

    #use Data::Dumper;
    #die Dumper($categories);

    my @root_cats = split /-/, $categories->{0};

#    use Data::Dumper;
#    die Dumper(\@root_cats);    

    foreach (@root_cats) {

        if (length($cat_names->{$_}) < 3) { next; }      

       # go through the root categories...
        make_cat_sub($cat_names->{$_},"0");
        print "Added cat: " . $cat_names->{$_} . "\n";
     
        my @sub_level = split /-/, $categories->{$_};

        foreach my $sl (@sub_level) {

           make_cat_sub($cat_names->{$_}."/".$cat_names->{$sl},$sl);
           print "Added sub-cat: " . $cat_names->{$_} . "/" . $cat_names->{$sl} . "\n";           


           my @sub_level2 = split /-/, $categories->{$sl};
 
           foreach my $sl2 (@sub_level2) {

              make_cat_sub($cat_names->{$_}."/".$cat_names->{$sl}."/".$cat_names->{$sl2},$sl2);
              print "Added sub-sub-cat: " . $cat_names->{$_} . "/" . $cat_names->{$sl} . "/". $cat_names->{$sl2} . "\n";           

              my @sub_level3 = split /-/, $categories->{$sl2};
      
              foreach my $sl3 (@sub_level3) {

                 make_cat_sub($cat_names->{$_}."/".$cat_names->{$sl}."/".$cat_names->{$sl2}."/".$cat_names->{$sl3},$sl3);
                 print "Added sub-sub-cat: " . $cat_names->{$_} . "/" . $cat_names->{$sl} . "/". $cat_names->{$sl2} ."/". $cat_names->{$sl3} . "\n";           

                  my @sub_level4 = split /-/, $categories->{$sl3};
      
                  foreach my $sl4 (@sub_level4) {

                    make_cat_sub($cat_names->{$_}."/".$cat_names->{$sl}."/".$cat_names->{$sl2}."/".$cat_names->{$sl3}."/".$cat_names->{$sl4},$sl4);
                    print "Added sub-sub-cat: " . $cat_names->{$_} . "/" . $cat_names->{$sl} . "/". $cat_names->{$sl2} ."/". $cat_names->{$sl3} ."/". $cat_names->{$sl4} . "\n";           

                  }
              }
           }
        }
    }    


# passes through to make_cat(), with some minor tweaks/changes..
sub make_cat_sub {

          my @_split_cat = split /\//, $_[0]; 
          my $catID      = $_[1];

          my $cnt_count = 0;
          my $category_name;
          foreach my $cat_loop (@_split_cat) {

             if (!$cnt_count) {
               $category_name  = $_split_cat[0];   
               $cnt_count++;
             } else {
               $category_name .= "/" . $cat_loop;
             }

             print "$category_name \n" if $DEBUG > 0;

            # make category....
             make_cat($category_name,$catID);

          } # end 'foreach'

}


# routine to make the categories....
sub make_cat {

   my $cat   = $_[0];
   my $catID = $_[1];

   my $name = "";
   my @cut_cat = split /\//, $cat;
   for (my $i = 0; $i < $#cut_cat; $i++) {
      $name .= $cut_cat[$i] . "/";    
   }

   my $cat_name = $cut_cat[$#cut_cat];

   $name =~ s,^/,,;
   $name =~ s,/$,,;
   $cat_name =~ s,^/,,;
   $cat_name =~ s,/$,,;

   my $fatherID = $DB->table('Category')->select( ['ID'], { Full_Name => $name } )->fetchrow || 0;

   if (!$DB->table('Category')->count( { Full_Name => $cat } )) {

      # now we can do the actual adding of the category...
        my $table = $DB->table ('Category');
           $table->insert ( {   Name => $cat_name,
                                FatherID => $fatherID,
                                Number_of_Links => 0,
                                Description => 'SUB1',
                                Has_New_Links => "No",
                                Has_Changed_Links => "No",
                                Timestmp => \"NOW()",
                                Full_Name => $cat,
                                ALLPOSTERS_ID => $catID                                
                            } ) or die $GT::SQL::error, " \nCat: $cat \n";   
        
      print ">> $name\n";

   } else {

      print "<< $name\n" if $DEBUG > 0;

   }
   

}

############################################################
############################################################
############################################################
# Now lets actually import the posters themselves...

  # show a message, so we know what is going on =)
   print "\n\n>>> IMPORTING POSTERS NOW... <<<\n\n";

  # lets get ALL the category/image relations put into a quick to access hashref
   my $relations;
   my $count = 0;
   open(CATSREAD, "$category_prod_info_file") || die "Unable to open $category_prod_info_file. Reason: $!";
     while (<CATSREAD>) {
      chomp;
      $count++;
      unless ($count % 10000) { print "$count\n"; }
      my ($CategoryNumber,$ProductNumber,$Rank) = split /\|/, $_;
      $relations->{$ProductNumber} = $CategoryNumber;
     }
   close(CATSREAD);



   open(CATSREAD, "$image_details_file") || die "Unable to open $image_details_file. Reason: $!";
     while (<CATSREAD>) {

      if ($_ =~ /\QArtist/) { next; }

      my ($Artist, $CanBeFramed, $CanBeMounted, $DateAdded, $Directory, $FileName, $ImageHeight, $ImageURL, $ImageWidth, $ListPrice, $OurPrice, $ProductHeight, $ProductLink, $ProductType, $ProductWidth, $QuickFrameable, $Rank, $ThumbnailHeight, $ThumbnailWidth, $ProductNumber, $Title, $FramedLink, $MountedLink) = split /\|/, $_;

      my $cat_ID = $DB->table('Category')->select( ['ID'],{ ALLPOSTERS_ID => $relations->{$ProductNumber} } )->fetchrow;

      if ($cat_ID < 1) {
         print "Invalid categoryID: $relations->{$ProductNumber} ($ProductNumber) SKIPPING \n" if $DEBUG > 1;
         next;
      }

      my $rec;

      $ListPrice =~ s/00$//; # get rid of extra 0's for some reason :/
      $OurPrice =~ s/00$//; # get rid of extra 0's for some reason :/
 
      $ProductLink =~ s/\[REPLACE\_WEBSITEID\]/$ALLPOSTERSID/;

      $rec->{Title} = $Title;
      $rec->{URL}   = $ProductLink;
      $rec->{Contact_Email} = 'foo@fo-1test.com';
      $rec->{Description} = '';
      $rec->{LinkOwner} = 'ALLPOSTERS';
      $rec->{Hits} = '0';
      $rec->{Status} = '0';
      $rec->{isPopular} = 'No';
      $rec->{Add_Date} = GT::Date::date_get();
      $rec->{Rating} = '0';
      $rec->{Votes} = '0';
      $rec->{Mod_Date} = GT::Date::date_get();
      $rec->{isValidated} = 'Yes';
      
      $rec->{Artist} = $Artist;
      $rec->{CanBeFramed} = $CanBeFramed;
      $rec->{CanBeMounted} = $CanBeMounted;
      $rec->{ImageHeight} = $ImageHeight;
      $rec->{ImageWidth} = $ImageWidth;
      $rec->{ImageURL} = $ImageURL;
      $rec->{ListPrice} = $ListPrice;
      $rec->{OurPrice} = $OurPrice;
      $rec->{ProductHeight} = $ProductHeight;
      $rec->{ProductWidth} = $ProductWidth;
      $rec->{ProductLink} = $ProductLink;
      $rec->{ProductType} = $ProductType;
      $rec->{QuickFrameable} = $QuickFrameable;
      $rec->{ThumbnailHeight} = $ThumbnailHeight;
      $rec->{ThumbnailWidth} = $ThumbnailWidth;
      $rec->{ProductNumber}  = $ProductNumber;

      $rec->{'CatLinks.CategoryID'} = $cat_ID;

      if (!$Title) { next; }

      $DB->table('Links')->add($rec) || die $GT::SQL::error;

      print "Added $rec->{Title} \n";

     }
   close(CATSREAD);


   print "\nCleaning out empty categories...\n";   

   # clean out the tables before doing an import...
    my $sth = $DB->table('Category')->select( ['ID'], GT::SQL::Condition->new('Number_of_Links','=','0') ) || die $GT::SQL::error;
    while (my $hit = $sth->fetchrow_hashref) {
      $DB->table('Category')->delete( { ID => $hit->{ID} } );
      print "Deleting category: $hit->{ID} (no posters) \n";
    }

   print "\n\n\nALL DONE!\n\n\n";

